==> ui-canvas-0.1.4-sources.jar: commonMain/com/cerve/development/ui/canvas/model/CerveCanvasState.kt <==
package com.cerve.development.ui.canvas.model

import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.setValue
import androidx.compose.runtime.toMutableStateList
import com.cerve.development.ui.canvas.operators.CerveCanvasDefaults
import kotlinx.serialization.Serializable

@Serializable
data class CerveCanvasState(
    private val initialLines: CerveCanvasLineData = CerveCanvasDefaults.canvasLines,
    private val initialEraserProperties: CerveCanvasEraserProperties = CerveCanvasDefaults.canvasEraserProperties,
    private val initialTransformations: CerveCanvasTransformations = CerveCanvasDefaults.canvasTransformations,
    private val initialInteractionType: CerveCanvasInteractionType = CerveCanvasInteractionType.None
) {

    var eraserProperties by mutableStateOf(initialEraserProperties)
        private set

    var gridLineCount by mutableStateOf(initialLines.gridLineCount)
        private set

    var transformations by mutableStateOf(initialTransformations)
        private set

    var interactionType by mutableStateOf(initialInteractionType)
        private set

    var drawnLines: MutableList<CerveLine> = initialLines.drawnLines.toMutableStateList()
    val drawnHistoryLines: MutableList<CerveLine> = initialLines.drawnHistoryLines.toMutableStateList()
    var drawnLineCandidates: MutableList<CerveLine> = initialLines.drawnLineCandidates.toMutableStateList()

    val selectedCells: MutableList<CerveCell> = initialLines.selectedCells.toMutableStateList()

    fun onUpdateEraserProperties(
        value: CerveCanvasEraserProperties
    ) { eraserProperties = value }

    fun onUpdateGridLineCount(
        value: Int
    ) { gridLineCount = value }

    fun onUpdateTransformations(
        value: CerveCanvasTransformations?
    ) { transformations = value ?: initialTransformations }

    fun onChangeInteractionType(
        value: CerveCanvasInteractionType
    ) { interactionType = value }

    fun onUndo() {
        val removed = drawnLines.removeLastOrNull()
        removed?.let { drawnHistoryLines.add(removed) }
    }

    fun onRedo() {
        val history = drawnHistoryLines.removeLastOrNull()
        history?.let { drawnLines.add(history) }
    }

    fun onResetDrawnLines(
        value: List<CerveLine>
    ) { drawnLines = value.toMutableStateList() }

    fun onResetDrawnLineCandidate(
        value: List<CerveLine>
    ) { drawnLineCandidates = value.toMutableStateList() }

    fun onClear() {
        drawnHistoryLines.clear()
        drawnLines.clear()
        drawnLineCandidates.clear()
    }

}

==> ui-canvas-0.1.6-sources.jar: commonMain/com/cerve/development/ui/canvas/model/CerveCanvasState.kt <==
package com.cerve.development.ui.canvas.model

import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.setValue
import androidx.compose.runtime.snapshots.SnapshotStateList
import androidx.compose.runtime.toMutableStateList
import com.cerve.development.ui.canvas.operators.CerveCanvasDefaults
import kotlinx.serialization.Serializable
import kotlinx.serialization.Transient

@Serializable
data class CerveCanvasState(
    private val initialLines: CerveCanvasLineData = CerveCanvasDefaults.canvasLines,
    private val initialEraserProperties: CerveCanvasEraserProperties = CerveCanvasDefaults.canvasEraserProperties,
    private val initialTransformations: CerveCanvasTransformations = CerveCanvasDefaults.canvasTransformations,
    private val initialInteractionType: CerveCanvasInteractionType = CerveCanvasInteractionType.None
) {

    var eraserProperties by mutableStateOf(initialEraserProperties)
        private set

    var gridLineCount by mutableStateOf(initialLines.gridLineCount)
        private set

    var transformations by mutableStateOf(initialTransformations)
        private set

    var interactionType by mutableStateOf(initialInteractionType)
        private set

    @Transient
    val drawnLines = initialLines.drawnLines.toMutableStateList()
    @Transient
    val drawnHistoryLines = initialLines.drawnHistoryLines.toMutableStateList()
    @Transient
    val drawnLineCandidates = initialLines.drawnLineCandidates.toMutableStateList()

    val selectedCells: MutableList<CerveCell> = initialLines.selectedCells.toMutableStateList()

    fun onUpdateEraserProperties(
        value: CerveCanvasEraserProperties
    ) { eraserProperties = value }

    fun onUpdateGridLineCount(
        value: Int
    ) { gridLineCount = value }

    fun onUpdateTransformations(
        value: CerveCanvasTransformations?
    ) { transformations = value ?: initialTransformations }

    fun onChangeInteractionType(
        value: CerveCanvasInteractionType
    ) { interactionType = value }

    fun onUndo() {
        val removed = drawnLines.removeLastOrNull()
        removed?.let { drawnHistoryLines.add(removed) }
    }

    fun onRedo() {
        val history = drawnHistoryLines.removeLastOrNull()
        history?.let { drawnLines.add(history) }
    }

    fun onResetDrawnLines(
        value: List<CerveLine>
    ) {
        drawnLines.clear()
        drawnLines.addAll(value)
    }

    fun onResetDrawnLineCandidate(
        value: List<CerveLine>
    ) {
        drawnLineCandidates.clear()
        drawnLineCandidates.addAll(value)
    }

    fun onClear() {
        drawnHistoryLines.clear()
        drawnLines.clear()
        drawnLineCandidates.clear()
    }

}
